/*
 * Laser Game V1.3.cpp
 *
 * Created: 15.01.2016 20:51:04
 * Author : Martin Dank
 */ 

#define F_CPU 8000000UL

//definice pro pomocnou ("auxilliary") RGB led diodu
#define ARGB_RED_ON	      PORTD |=(1<<PORTD2)
#define ARGB_BLUE_ON	    PORTD |=(1<<PORTD0)
#define ARGB_GREEN_ON	    PORTD |=(1<<PORTD1)

#define ARGB_RED_OFF	    PORTD &=~(1<<PORTD2)
#define ARGB_BLUE_OFF	    PORTD &=~(1<<PORTD0)
#define ARGB_GREEN_OFF	  PORTD &=~(1<<PORTD1)

#define ARGB_ALL_OFF	    ARGB_BLUE_OFF; ARGB_GREEN_OFF; ARGB_RED_OFF

//definice pro hlavn ("main") RGB led diody
#define MRGB_RED_ON		   PORTC |=(1<<PORTC3)
#define MRGB_BLUE_ON	   PORTC |=(1<<PORTC2)
#define MRGB_GREEN_ON	   PORTC |=(1<<PORTC1)

#define MRGB_RED_OFF	   PORTC &=~(1<<PORTC3)
#define MRGB_BLUE_OFF	   PORTC &=~(1<<PORTC2)
#define MRGB_GREEN_OFF	 PORTC &=~(1<<PORTC1)

#define MRGB_ALL_OFF	   MRGB_BLUE_OFF; MRGB_GREEN_OFF; MRGB_RED_OFF

//definice pro pulzn LED
#define PLED_ON			     PORTD |=(1<<PORTD6)
#define PLED_OFF		     PORTD &=~(1<<PORTD6)

//definice pro infraervenou LED
#define IRLED_ON		     PORTD |=(1<<PORTD7)
#define IRLED_OFF		     PORTD &=~(1<<PORTD7)

//definice pro piezo speaker
#define SPK_ON			     PORTD |=(1<<PORTD5)
#define SPK_OFF			     PORTD &=~(1<<PORTD5)

//kontrola tlatek
#define AUX1_PRESSED	   !(PINB & (1<<PINB2))
#define AUX2_PRESSED	   !(PINB & (1<<PINB1))
#define TRIG_PRESSED	   !(PIND & (1<<PIND0))

//definice pro IR receiver
#define RECEIVE_LOW		   !(PIND & (1<<PIND3))

//definice pro ADC pevodnk a men napt baterie
#define CUTOFF          3.00
unsigned char overflowCount;

//definice pro penos
#define CONTROL_BIT		  2400
#define HIGH_BIT		    1200
#define LOW_BIT			    600
#define SPACE			      600
const bool codeAlly[] = {1, 0, 0, 1};
const bool codeEnemy[] = {0, 1, 1, 0};
const bool codeRevive[] = {1, 0, 1, 0};
bool codeReceived[4];
bool playerAlive = true;

float receiveTime[6];
bool timerStarted;
unsigned char arrayPosition;


//definice td a funkc
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <string.h>

#include "melody.h"
#include "test_sequence.h"

void IRWrite(unsigned int duration);
void SendData();


int main(void)
{
  DDRD |= (1<<DDD0)|(1<<DDD1)|(1<<DDD2);		//nastav vstupy na D0 a D2 pro ARGB
	DDRC |= (1<<DDC1)|(1<<DDC2)|(1<<DDC3);		//nastav vstupy na C1 a C3 pro MRGB
	DDRD |= (1<<DDD5)|(1<<DDD6)|(1<<DDD7);		//nastav vstupy na D5 a D7 pro piezo bzuk, IR LED a pulzn LED
	
	PORTB |=(1<<PORTB1)|(1<<PORTB2);			    //zapne intern pullup resitory pro tlatka AUX1 a AUX2
	PORTD |=(1<<PORTD0);						          //zapne intern pullup resistory pro tlatko TRIG
		
	EIMSK |= (1<<INT1);						          	//povol peruen INT1
	EICRA |= (1<<ISC10);						          //nastav INT1 na jakoukoliv logickou zmnu
	TIMSK1 |= (1<<TOIE1);						          //povol peruen pi peteen asovae 1
	TCCR2B |= (1<<CS20) | (1<< CS21) | (1<<CS22);	//nastav prescaler asovae 2 na 1024
	TIMSK2 |= (1<<TOIE2);						          //povol peruen pi peteen asovae 2
	sei();										                //globln povol peruen na celm mikrokontroleru		
	
	ADMUX = (1<<REFS0);							          //nastav referenci pro men napt baterie
	ADCSRA = (1<<ADEN);							          //povol ADC
	ADCSRA |= (1<<ADPS1) | (1<<ADPS2);			  //nastav frekvenci ADC na 125 kHz
	
	melodyStart();							            	//tikrt zelen zablik a zahraje vodn melodii
	ARGB_GREEN_ON;
	MRGB_GREEN_ON;
	_delay_ms(100);
	ARGB_GREEN_OFF;
	MRGB_GREEN_OFF;
	_delay_ms(100);
	
	ARGB_GREEN_ON;
	MRGB_GREEN_ON;
	_delay_ms(100);
	ARGB_GREEN_OFF;
	MRGB_GREEN_OFF;
	_delay_ms(100);
	
	ARGB_GREEN_ON;
	MRGB_GREEN_ON;
	_delay_ms(100);
	ARGB_GREEN_OFF;
	MRGB_GREEN_OFF;
	_delay_ms(100);
	
	ARGB_GREEN_ON;
	MRGB_GREEN_ON;
	
	
	while (1) 
    {
		if (TRIG_PRESSED)						           //pokud je zmknuto tlatko 1 ("spou")...
		{
			if (playerAlive == true)			       //...a hr je zrove naivu...
			{
				PLED_ON;						               //...zbra vystel a d o tom hri vdt
				ARGB_ALL_OFF;
				ARGB_BLUE_ON;
				SendData();
				PLED_OFF;
				melodyFire();
				ARGB_BLUE_OFF;
				
			}
			else
			{
				melodyInvalid();			             //pokud hr nen naivu, neme stlet
			}
			
			while (TRIG_PRESSED)				         //hr mus spou pustit aby mohl znovu vystelit
			{
				_delay_ms(1);
			}
			ARGB_GREEN_ON;
		}
		
		if (AUX1_PRESSED)
		{
			//akce pomocnho tlatka 1
			while (AUX1_PRESSED)
			{
				_delay_ms(1);
			}
		}
		
		if (AUX2_PRESSED)
		{
			//akce pomocnho tlaka 2
			while (AUX2_PRESSED)
			{
				_delay_ms(1);
			}

		}
		
		
	
    }
}

void IRWrite(unsigned int duration)				//slou k samotn modulaci IR LED diody na frekvenci 38 kHz
{
	for (unsigned int i = 0; i <= duration/26; i++)
	{
		IRLED_ON;
		_delay_us(13);
		IRLED_OFF;
		_delay_us(13);
	}
}

void SendData(void)								//d odesln dat pomoc IR LED diody
{
	IRWrite(CONTROL_BIT);
	_delay_us(SPACE);
		
	for (int i = 0; i < 4; i++)
	{
		if (codeAlly[i] == 0)
		{
			IRWrite(LOW_BIT);
		}
		else
		{
			IRWrite(HIGH_BIT);
		}
		_delay_us(SPACE);
	}
	
	IRWrite(CONTROL_BIT);
	
}



ISR(INT1_vect)									//peruen pokud dojde k logick zmn na INT1
{
	if (timerStarted == false)					//pokud se jedn o pjem prvnho signlu, zapne asova 1
	{
		timerStarted = true;
		TCCR1B |= (1 << CS10);					// nastavit prescaler asovae 1 na 1 (tzn. asova je asovn frekvenc 8 MHz)
	}
	else
	{
		if (RECEIVE_LOW)						//pokud nyn pjmme logickou 0, resetovat tac registr asovae 1
		{
			TCNT1 = 0;
		}
		else
		{
			//pokud nyn pjmme logickou 1, pest hodnotu tacho registru a pevst ji na as v mikrosekundch
			receiveTime[arrayPosition] = (TCNT1 * 0.125);
			arrayPosition++;
			TCNT1 = 0;
		}	
	}
}

ISR(TIMER1_OVF_vect)							//pokud dojde k peteen tacho registru
{
	TCCR1B = 0;									//zastavit asova
	timerStarted = false;
	arrayPosition = 0;

	
	if (!(receiveTime[0] > (CONTROL_BIT - 100)) || !(receiveTime[5] > (CONTROL_BIT - 100)))
	{
		//zkontroluje, jestli prvn a posledn sti pjmu jsou kontroln bity a zdali jsou v podku; pokud ne, provede nsledujc akci
		MRGB_BLUE_ON;
		_delay_ms(300);
		MRGB_BLUE_OFF;
	}
	else
	{
		//pevede pijat asy na hodnoty logick 1 nebo 0
		for (int i = 1; i < 5; i ++)
		{
			if (receiveTime[i] < 1100)
			{
				codeReceived[arrayPosition] = 0;
				arrayPosition++;
			}
			else if (receiveTime[i] > 1100)
			{
				codeReceived[arrayPosition] = 1;
				arrayPosition++;
			}
		}
		
		
		if ((codeReceived[0] == codeEnemy[0]) && (codeReceived[1] == codeEnemy[1]) && (codeReceived[2] == codeEnemy[2]) && (codeReceived[3] == codeEnemy[3]))
		{
			//pokud je pijat kd neptelskm kdem, "zabije" hre
			if (playerAlive == true)
			{
				playerAlive = false;
				MRGB_ALL_OFF;
				MRGB_RED_ON;
				ARGB_ALL_OFF;
				ARGB_RED_ON;
				melodyHit();
			}
		}
		else if ((codeReceived[0] == codeRevive[0]) && (codeReceived[1] == codeRevive[1]) && (codeReceived[2] == codeRevive[2]) && (codeReceived[3] == codeRevive[3]))
		{
			//pokud je pijat kd oivujvm kdem, oiv hre
			playerAlive = true;
			MRGB_GREEN_ON;
			ARGB_GREEN_ON;
			melodyStart();
		}
		else
		{
			melodyInvalid();
		}
		
	}
	
	//zresetuje nkter promnn
	memset(receiveTime, 0, 6);
	memset(codeReceived, 0, 4);
	arrayPosition = 0;
	 
}

ISR (TIMER2_OVF_vect)
//slou k pravidelnmu men napt baterie
{
	overflowCount++;
	
	if (overflowCount > 200)
	{
		overflowCount = 0;
		ADCSRA |= (1<<ADSC);					//zahj innost ADC
		while(ADCSRA & (1<<ADSC));				//pok na dokonen konverze
		
		if ((ADC * 0.0048828125) < CUTOFF)		//pokud je napt baterie ni ne CUTOFF
		{
			ARGB_ALL_OFF;
			MRGB_ALL_OFF;
			cli();
			while (1)							//pevede zazen do nekonen smyky s pouze blikajc ervenou diodou
			{
				ARGB_RED_ON;
				_delay_ms(500);
				ARGB_RED_OFF;
				_delay_ms(500);
			}
			
		}
	}
	
}





